<?php
/**
 * A general pool class
 *
 * @author		Roland Haeder <webmaster@shipsimu.org>
 * @version		0.0.0
 * @copyright	Copyright (c) 2007, 2008 Roland Haeder, 2009 - 2015 Hub Developer Team
 * @license		GNU GPL 3.0 or any newer version
 * @link		http://www.shipsimu.org
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
class BasePool extends BaseHubSystem implements Visitable {
	/**
	 * Socket array elements
	 */
	const SOCKET_ARRAY_RESOURCE  = 'resource';
	const SOCKET_ARRAY_CONN_TYPE = 'connection_type';

	/**
	 * A list of pool entries
	 */
	private $poolEntriesInstance = NULL;

	/**
	 * An array with all valid connection types
	 */
	private $connectionTypes = array();

	/**
	 * Protected constructor
	 *
	 * @param	$className	Name of the class
	 * @return	void
	 */
	protected function __construct ($className) {
		// Call parent constructor
		parent::__construct($className);

		// Init the pool entries
		$this->poolEntriesInstance = ObjectFactory::createObjectByConfiguredName('pool_entries_list_class');

		// Init array of connection types
		$this->connectionTypes = array(
			BaseConnectionHelper::CONNECTION_TYPE_INCOMING,
			BaseConnectionHelper::CONNECTION_TYPE_OUTGOING,
			BaseConnectionHelper::CONNECTION_TYPE_SERVER
		);
	}

	/**
	 * Getter for pool entries instance
	 *
	 * @return	$poolEntriesInstance	An instance for pool entries (list)
	 */
	public final function getPoolEntriesInstance () {
		return $this->poolEntriesInstance;
	}

	/**
	 * Adds an instance to a pool segment
	 *
	 * @param	$group				Name of the pool group
	 * @param	$poolSegment		Name of the pool segment
	 * @param	$visitableInstance	An instance of a class that should bed added to the pool
	 * @return	void
	 */
	protected final function addInstance ($group, $poolName, Visitable $visitableInstance) {
		// Debug message
		//* NOISY-DEBUG: */ self::createDebugInstance(__CLASS__)->debugOutput('POOL[' . __METHOD__ . ':' . __LINE__ . ']: group=' . $group . ',poolName=' . $poolName . ',visitableInstance=' . $visitableInstance->__toString() . ' - CALLED!');

		// Make sure the group is not 'invalid'
		assert($group != 'invalid');

		// Is the pool group there?
		if (!$this->getPoolEntriesInstance()->isGroupSet($group)) {
			// Create the missing pool group
			$this->getPoolEntriesInstance()->addGroup($group);
		} // END - if

		// Add it to given pool group
		$this->getPoolEntriesInstance()->addInstance($group, $poolName, $visitableInstance);

		// Debug message
		//* NOISY-DEBUG: */ self::createDebugInstance(__CLASS__)->debugOutput('POOL[' . __METHOD__ . ':' . __LINE__ . ']: EXIT!');
	}

	/**
	 * Adds an entry to the pool
	 *
	 * @param	$poolEntry	The new pool entry that should be added
	 * @return	void
	 */
	protected final function addPoolEntry ($poolEntry) {
		$this->getPoolEntriesInstance()->addEntry('pool', $poolEntry);
	}

	/**
	 * Accepts the visitor to process the visit "request"
	 *
	 * @param	$visitorInstance	An instance of a Visitor class
	 * @return	void
	 */
	public function accept (Visitor $visitorInstance) {
		// Debug message
		//* NOISY-DEBUG: */ self::createDebugInstance(__CLASS__)->debugOutput('POOL[' . __METHOD__ . ':' . __LINE__ . ']: ' . $visitorInstance->__toString() . ' has visited - CALLED!');

		// Visit this pool
		$visitorInstance->visitPool($this);

		// Do we have a registry instance for this visitor's iterator?
		if (Registry::getRegistry()->instanceExists($visitorInstance->getVisitorMode() . '_pool_iterator')) {
			// Get the instance from registry
			$iteratorInstance = Registry::getRegistry()->getInstance($visitorInstance->getVisitorMode() . '_pool_iterator');
		} else {
			// Get a new iterator instance
			$iteratorInstance = ObjectFactory::createObjectByConfiguredName($visitorInstance->getVisitorMode() . '_pool_iterator_class', array($this->getPoolEntriesInstance()));

			// Add it to the registry
			Registry::getRegistry()->addInstance($visitorInstance->getVisitorMode() . '_pool_iterator', $iteratorInstance);
		}

		// Reset the counter
		$iteratorInstance->rewind();

		// Visit all registered entries
		while ($iteratorInstance->valid()) {
			// Get current entry
			$poolEntry = $iteratorInstance->current();

			// Is this entry visitable?
			if ($poolEntry instanceof Visitable) {
				// Visit this entry as well
				//* NOISY-DEBUG: */ self::createDebugInstance(__CLASS__)->debugOutput('BASE-POOL[' . __METHOD__ . ':' . __LINE__ . ']: Going to visit pooled object ' . $poolEntry->__toString() . ' with visitor ' . $visitorInstance->__toString() . ' ...');
				$poolEntry->accept($visitorInstance);
			} else {
				// Cannot visit this entry
				$this->partialStub('Pool entry with key ' . $iteratorInstance->key() . ' has improper type ' . gettype($poolEntry) . ', reason: not implementing Visitable.');
			}

			// Advance to next entry
			$iteratorInstance->next();
		} // END - while

		// Debug message
		//* NOISY-DEBUG: */ self::createDebugInstance(__CLASS__)->debugOutput('POOL[' . __METHOD__ . ':' . __LINE__ . ']: ' . $visitorInstance->__toString() . ' has visited - EXIT!');
	}

	/**
	 * Gets the array from specified list
	 *
	 * @param	$list	The list identifier we should return
	 * @return	$array	The requested array
	 */
	protected final function getArrayFromList ($list) {
		// Get the array
		$array = $this->getPoolEntriesInstance()->getArrayFromList($list);

		// Return it
		return $array;
	}

	/**
	 * Checks whether the given connection type is valid
	 *
	 * @param	$connectionType		Type of connection, can be 'incoming', 'outgoing' or 'server'
	 * @return	$isValid			Whether the provided connection type is valid
	 */
	protected function isValidConnectionType ($connectionType) {
		// Is it valid?
		$isValid = in_array($connectionType, $this->connectionTypes, TRUE);

		// Return result
		return $isValid;
	}
}

// [EOF]
?>
